const config = require('config')
const { get } = require('lodash')

const {
  User,
  Email,
  services,
  Fragment,
} = require('pubsweet-component-helper-service')

const { getEmailCopy } = require('./emailCopy')

const unsubscribeSlug = config.get('unsubscribe.url')
const editorialAssistantEmail = config.get('mailer.editorialAssistant')

module.exports = {
  async sendNotifications({
    baseUrl,
    fragment,
    UserModel,
    collection,
    isNewVersion = false,
    isTechnicalCheck,
    isMajorRecommendation,
  }) {
    const fragmentHelper = new Fragment({ fragment })
    const parsedFragment = await fragmentHelper.getFragmentData({
      handlingEditor: collection.handlingEditor,
    })
    const { submittingAuthor } = await fragmentHelper.getAuthorData({
      UserModel,
    })

    const subjectBaseText = `${collection.customId}: Manuscript`
    const titleText = `the manuscript titled "${parsedFragment.title}" by ${
      submittingAuthor.firstName
    } ${submittingAuthor.lastName}`

    const email = new Email({
      type: 'user',
      content: {
        signatureName: get(collection, 'handlingEditor.name', 'Hindawi'),
        ctaLink: services.createUrl(
          baseUrl,
          `/projects/${collection.id}/versions/${fragment.id}/details`,
        ),
        ctaText: 'MANUSCRIPT DETAILS',
      },
    })

    const userHelper = new User({ UserModel })
    const eicName = await userHelper.getEiCName()
    if (isNewVersion) {
      sendHandlingEditorEmail({
        email,
        eicName,
        baseUrl,
        titleText,
        subjectBaseText,
        handlingEditor: get(collection, 'handlingEditor', {}),
      })
    }

    if (isMajorRecommendation) {
      sendReviewersEmail({
        email,
        baseUrl,
        titleText,
        UserModel,
        fragmentHelper,
        subjectBaseText,
      })
    }

    if (isTechnicalCheck) {
      sendEQSEmail({
        email,
        eicName,
        baseUrl,
        collection,
        subjectBaseText,
      })
    }
  },
}

const sendHandlingEditorEmail = ({
  email,
  eicName,
  baseUrl,
  titleText,
  handlingEditor,
  subjectBaseText,
}) => {
  const emailType = 'he-new-version-submitted'

  email.toUser = {
    email: handlingEditor.email,
    name: handlingEditor.name,
  }
  email.content.unsubscribeLink = services.createUrl(baseUrl, unsubscribeSlug, {
    id: handlingEditor.id,
  })
  email.content.signatureName = eicName
  email.content.subject = `${subjectBaseText} Update`

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      titleText,
    }),
  })
  email.sendEmail({ html, text })
}

const sendReviewersEmail = async ({
  email,
  baseUrl,
  titleText,
  UserModel,
  fragmentHelper,
  subjectBaseText,
}) => {
  email.content.subject = `${subjectBaseText} Update`
  const emailType = 'submitted-reviewers-after-revision'

  const reviewers = await fragmentHelper.getReviewers({
    UserModel,
    type: 'submitted',
  })

  reviewers.forEach(reviewer => {
    email.toUser = {
      email: reviewer.email,
      name: `${reviewer.firstName} ${reviewer.lastName}`,
    }

    email.content.unsubscribeLink = services.createUrl(
      baseUrl,
      unsubscribeSlug,
      {
        id: reviewer.id,
      },
    )

    const { html, text } = email.getBody({
      body: getEmailCopy({
        emailType,
        titleText,
        expectedDate: services.getExpectedDate({ daysExpected: 14 }),
      }),
    })

    email.sendEmail({ html, text })
  })
}

const sendEQSEmail = ({
  email,
  eicName,
  baseUrl,
  collection,
  subjectBaseText,
}) => {
  const emailType = 'eqs-manuscript-submitted'

  email.toUser = {
    email: editorialAssistantEmail,
    name: 'Editorial Assistant',
  }

  email.content.unsubscribeLink = baseUrl
  email.content.signatureName = eicName
  email.content.subject = `${subjectBaseText} Submitted`
  email.content.ctaLink = services.createUrl(
    baseUrl,
    config.get('eqs-decision.url'),
    {
      collectionId: collection.id,
      customId: collection.customId,
      token: collection.technicalChecks.token,
    },
  )
  email.content.ctaText = 'MAKE DECISION'

  const { html, text } = email.getBody({
    body: getEmailCopy({
      emailType,
      customId: collection.customId,
    }),
  })
  email.sendEmail({ html, text })
}
